import React, { useContext, useCallback, useState } from 'react';
import { Editor, Range, Node, Text, Path } from 'slate';
import { useSlate, ReactEditor } from 'slate-react';

const isHighlightEditor = (editor) => 'insertData' in editor && 'setFragmentData' in editor && 'history' in editor && 'undo' in editor && 'redo' in editor;
const useHighlightSlate = (editor) => {
    if (!isHighlightEditor(editor)) {
        throw new Error('The slate editor type isn\'t assigned to HighlightEditor type.');
    }
    return editor;
};

const isHighlightLeaf = (leaf) => 'select' in leaf;
const wrapperClassNameMapping = {
    SELECTED: 'highlight-leaf-selected',
    DESELECTED: 'highlight-leaf-deselected',
};
const HighlightLeaf = (props) => {
    const { children, leaf, attributes } = props;
    if (!isHighlightLeaf(leaf) || !leaf.select) {
        return React.createElement("span", Object.assign({}, attributes), children(leaf));
    }
    if (leaf.select === 'DESELECTED') {
        return (React.createElement("span", Object.assign({}, attributes, { className: wrapperClassNameMapping.DESELECTED }), children(leaf)));
    }
    return (React.createElement("span", Object.assign({}, attributes, { className: wrapperClassNameMapping.SELECTED }), children(leaf)));
};

const SelectedTextContext = React.createContext(null);
const UnDecorateListContext = React.createContext(null);
const defaultComparer = (preRanges, ranges) => {
    const getRangeString = (range) => {
        const { anchor, focus } = range;
        return `${anchor.path.join(',')}-${anchor.offset}-${focus.path.join(',')}-${focus.offset}`;
    };
    const map = new Map();
    preRanges.forEach((r) => map.set(getRangeString(r), Object.assign({}, r)));
    ranges.forEach((r) => {
        const rValue = map.get(getRangeString(r));
        return !!rValue ? map.set(getRangeString(r), Object.assign(Object.assign({}, rValue), r)) : map.set(getRangeString(r), Object.assign({}, r));
    });
    return Array.from(map.values());
};
const builtInMouseUpConstraint = (editor) => {
    const { selection } = editor;
    const { anchor: { path: aPath }, focus: { path: fPath }, } = selection;
    if (!Path.equals(aPath, fPath) || Editor.string(editor, editor.selection).includes('\n')) {
        throw new Error('Selection Constaint: Cross Paragraph Or Highlight Text Selection');
    }
};
const useSelection = () => {
    const selectedTextContext = useContext(SelectedTextContext);
    const unDecorateListContext = useContext(UnDecorateListContext);
    if (!selectedTextContext || !unDecorateListContext) {
        throw new Error('The `useSelection` hook must be used inside the <HighlightSlate> component\'s context.');
    }
    const editor = useHighlightSlate(useSlate());
    const [selectedText, setSelected] = selectedTextContext;
    const [unDecorateList, setUnDecorateList] = unDecorateListContext;
    /**
     * @todo
     * Deal with "Cross element error"
     */
    const createMouseUpHandler = useCallback(({ validator, callback }) => (event) => {
        if (validator && typeof validator === 'function' && !validator(event, editor))
            return;
        const { selection } = editor;
        const selected = Editor.string(editor, editor.selection);
        if (!selection || !ReactEditor.isFocused(editor) || Range.isCollapsed(selection) || !selected)
            return;
        if (selected === selectedText)
            return;
        /**
           * Reset 'unDecorateList' if re-selected
           */
        setUnDecorateList([]);
        setSelected(selected);
        if (callback && typeof callback === 'function')
            callback(event, editor);
    }, [editor, selectedText, setSelected, setUnDecorateList]);
    const mouseDownHandler = useCallback((event) => {
        const { target } = event;
        const highlightSelectedNode = target.closest(`.${wrapperClassNameMapping.SELECTED}`);
        if (!highlightSelectedNode)
            return;
        const slateTextDomNode = highlightSelectedNode.parentNode;
        const leafIdxInText = Array.from(slateTextDomNode.children).findIndex((el) => highlightSelectedNode.isSameNode(el));
        const leafSequenceIdx = leafIdxInText % 2 ? (leafIdxInText - 1) / 2 : leafIdxInText / 2;
        const slateTextNode = ReactEditor.toSlateNode(editor, slateTextDomNode);
        const parts = Node.string(slateTextNode).split(selectedText);
        const path = ReactEditor.findPath(editor, slateTextNode);
        const { range } = Array.from(Array(leafSequenceIdx + 2)).reduce((prev, _, idx) => {
            const newOffset = prev.offset + parts[idx].length + selectedText.length;
            /**
             * Pass directly if its first item
             */
            if (!idx) {
                return {
                    range: null,
                    offset: newOffset,
                };
            }
            const newRange = {
                anchor: { path, offset: prev.offset - selectedText.length },
                focus: { path, offset: prev.offset },
            };
            return {
                range: newRange,
                offset: newOffset,
            };
        }, {
            range: null,
            offset: 0,
        });
        setUnDecorateList((prev) => [...prev, range]);
    }, [selectedText, editor, setUnDecorateList]);
    const createMouseHandlers = useCallback((props) => ({
        onMouseUp: createMouseUpHandler({
            validator: (event, e) => {
                if ('validator' in props && typeof props.validator === 'function') {
                    return props.validator(builtInMouseUpConstraint, event, e);
                }
                try {
                    builtInMouseUpConstraint(e);
                }
                catch (err) {
                    return false;
                }
                return true;
            },
            callback: props.callback,
        }),
        onMouseDown: mouseDownHandler,
    }), [createMouseUpHandler, mouseDownHandler]);
    const createDecorate = useCallback(({ preDecorate, comparer }) => (entry) => {
        const preRanges = preDecorate && typeof preDecorate === 'function' ? preDecorate(entry) : [];
        const ranges = [];
        const [node, path] = entry;
        if (selectedText && Text.isText(node)) {
            const { text } = node;
            const parts = text.split(selectedText);
            let offset = 0;
            /**
               * Insert leaf between two 'parts'
               */
            parts.forEach((part, i) => {
                if (i !== 0) {
                    const range = {
                        anchor: { path, offset: offset - selectedText.length },
                        focus: { path, offset },
                    };
                    const unDecorateIdx = unDecorateList.findIndex((r) => Range.includes(r, range));
                    if (!~unDecorateIdx) {
                        ranges.push(Object.assign(Object.assign({}, range), { select: 'SELECTED' }));
                    }
                    else {
                        ranges.push(Object.assign(Object.assign({}, range), { select: 'DESELECTED' }));
                    }
                }
                offset = offset + part.length + selectedText.length;
            });
        }
        return comparer && typeof comparer === 'function' ? comparer(preRanges, ranges) : defaultComparer(preRanges, ranges);
    }, [selectedText, unDecorateList]);
    return {
        createDecorate,
        createMouseHandlers,
    };
};

/**
 *
 * @param props IHighlightSlate
 * @returns Initialized Provider JSX component
 */
const HighlightSlate = (props) => {
    const { selectedText, setSelectedTxt, children } = props;
    const [unDecorated, setUnDecorated] = useState([]);
    return (React.createElement(SelectedTextContext.Provider, { value: [selectedText, setSelectedTxt] },
        React.createElement(UnDecorateListContext.Provider, { value: [unDecorated, setUnDecorated] }, children)));
};

export { HighlightLeaf, HighlightSlate, SelectedTextContext, UnDecorateListContext, useHighlightSlate, useSelection, wrapperClassNameMapping };
//# sourceMappingURL=index.es.js.map
